@@ -1,12 +1,10 @@
-/*
- * $Id: config.c,v 1.6 1995/05/06 16:26:13 root Exp root $
- */
 #include <stdio.h>
 #include <termios.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <fcntl.h>
+#include <errno.h>
 
 #include "config.h"
 #include "emu.h"
@@ -19,10 +17,8 @@
 #include "kversion.h"
 
 #include "dos2linux.h"
+#include "priv.h"
 
-#ifdef __NetBSD__
-extern int errno;
-#endif
 
 /*
  * XXX - the mem size of 734 is much more dangerous than 704. 704 is the
@@ -32,8 +28,8 @@ extern int errno;
 
 
 struct debug_flags d =
-{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-/* d  R  W  D  C  v  X  k  i  s  m  #  p  g  c  w  h  I  E  x  M  n  P  r  S */
+{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+/* d  R  W  D  C  v  X  k  i  T  s  m  #  p  g  c  w  h  I  E  x  M  n  P  r  S */
 
 static void     check_for_env_autoexec_or_config(void);
 int     parse_debugflags(const char *s, unsigned char flag);
@@ -59,6 +55,8 @@ config_defaults(void)
     config.xms_size = 0;
     config.max_umb = 0;
     config.dpmi = 0;
+    config.secure = 1;  /* need to have it 'on', else user may trick it out
+                           via -F option */
     config.mathco = 1;
     config.mouse_flag = 0;
     config.mapped_bios = 0;
@@ -104,12 +102,17 @@ config_defaults(void)
     config.dualmon = 0;
     config.force_vt_switch = 0;
     config.speaker = SPKR_EMULATED;
-#if 0				/* This is too slow, but why? */
-    config.update = 54945;
-#else
-    config.update = 27472;
-#endif
-    config.freq = 18;		/* rough frequency */
+
+    /* The frequency in usec of the SIGALRM call (in signal.c) is
+     * equal to this value / 6, and thus is currently 9158us = 100 Hz
+     * The 6 (TIMER DIVISOR) is a constant of unknown origin
+     * NOTE: if you set 'timer 18' in config.dist you can't get anything
+     * better that 55555 (108Hz) because of integer math.
+     * see timer_interrupt_init() in init.c
+     */
+    config.update = 54945;	/* should be = 1E6/config.freq */
+    config.freq = 18;		/* rough frequency (real PC = 18.2065) */
+
     config.timers = 1;		/* deliver timer ints */
     config.keybint = 1;		/* keyboard interrupts */
  
@@ -138,6 +141,9 @@ config_defaults(void)
     config.detach = 0;		/* Don't detach from current tty and open
 				 * new VT. */
     config.debugout = NULL;	/* default to no debug output file */
+
+    config.pre_stroke =NULL;	/* defualt no keyboard pre-strokes */
+
     config.sillyint = 0;
     config.must_spare_hardware_ram = 0;
     memset(config.hardware_pages, 0, sizeof(config.hardware_pages));
@@ -169,7 +175,9 @@ config_defaults(void)
 static void 
 open_terminal_pipe(char *path)
 {
+    enter_priv_off();
     terminal_fd = DOS_SYSCALL(open(path, O_RDWR));
+    leave_priv_setting();
     if (terminal_fd == -1) {
 	terminal_pipe = 0;
 	error("ERROR: open_terminal_pipe failed - cannot open %s!\n", path);
@@ -215,6 +223,7 @@ open_Xmouse_pipe(char *path)
 void 
 config_init(int argc, char **argv)
 {
+    extern char *commandline_statements;
     int             c;
     char           *confname = NULL;
 
@@ -239,21 +248,25 @@ config_init(int argc, char **argv)
 
     opterr = 0;
     confname = CONFIG_FILE;
-    while ((c = getopt(argc, argv, "ABCcF:kM:D:P:VNtsgx:Km234e:E:dXY:Z:o:O")) != EOF) {
+    while ((c = getopt(argc, argv, "ABCcF:I:kM:D:P:VNtsgx:Km234e:E:dXY:Z:o:O")) != EOF) {
 	switch (c) {
 	case 'F':
-	    if (getuid()) {
+	    if (get_orig_uid()) {
 		FILE *f;
-		priv_off();
+		enter_priv_off();
 		f=fopen(optarg, "r");
-		priv_default();
+		leave_priv_setting();
 		if (!f) {
 		  fprintf(stderr, "Sorry, no access to configuration file %s\n", optarg);
 		  exit(1);
 		}
+		fclose(f);
 	    }
 	    confname = optarg;	/* someone reassure me that this is *safe*? */
 	    break;
+	case 'I':
+	    commandline_statements = optarg;
+	    break;
 	case 'd':
 	    if (config.detach)
 		break;
@@ -267,14 +280,14 @@ config_init(int argc, char **argv)
 	    dbg_fd = stderr;
 	    break;
 	case 'o':
-	    priv_off();
 	    config.debugout = strdup(optarg);
+	    enter_priv_off();
 	    dbg_fd = fopen(config.debugout, "w");
+	    leave_priv_setting();
 	    if (!dbg_fd) {
 		fprintf(stderr, "can't open \"%s\" for writing\n", config.debugout);
 		exit(1);
 	    }
-	    priv_default();
 	    break;
 	}
     }
@@ -286,12 +299,8 @@ config_init(int argc, char **argv)
     }
 #endif
 
-    priv_off();
-
     parse_config(confname);
 
-    priv_default();
-
     if (config.exitearly)
 	leavedos(0);
 
@@ -303,9 +312,10 @@ config_init(int argc, char **argv)
     optind = 0;
 #endif
     opterr = 0;
-    while ((c = getopt(argc, argv, "ABCcF:kM:D:P:v:VNtT:sgx:Km2345e:dXY:Z:E:o:O")) != EOF) {
+    while ((c = getopt(argc, argv, "ABCcF:I:kM:D:P:v:VNtT:sgx:Km2345e:dXY:Z:E:o:O")) != EOF) {
 	switch (c) {
 	case 'F':		/* previously parsed config file argument */
+	case 'I':
 	case 'd':
 	case 'o':
 	case 'O':
@@ -333,6 +343,7 @@ config_init(int argc, char **argv)
 #endif
 	    break;
 	case 'Y':
+#ifdef X_SUPPORT
 	    open_Xkeyboard_pipe(optarg);
 	    config.cardtype = CARD_MDA;
 	    config.mapped_bios = 0;
@@ -346,10 +357,13 @@ config_init(int argc, char **argv)
 	    config.vga = 0;	/* this flags BIOS graphics */
 	    config.usesX = 1;
 	    config.console_keyb = 1;
+#endif
 	    break;
 	case 'Z':
+#ifdef X_SUPPORT
 	    open_Xmouse_pipe(optarg);
 	    config.usesX = 1;
+#endif
 	    break;
 	case 'K':
 	    warn("Keyboard interrupt enabled...this is still buggy!\n");
@@ -367,9 +381,7 @@ config_init(int argc, char **argv)
 	    break;
 	case 'P':
 	    if (terminal_fd == -1) {
-		priv_off();
 		open_terminal_pipe(optarg);
-		priv_default();
 	    } else
 		error("ERROR: terminal pipe already open\n");
 	    break;
@@ -382,7 +394,7 @@ config_init(int argc, char **argv)
 	    break;
 	case 'v':
 	    config.cardtype = atoi(optarg);
-	    if (config.cardtype > 4)
+	    if (config.cardtype > 7)	/* keep it updated when adding a new card! */
 		config.cardtype = 1;
 	    g_printf("Configuring cardtype as %d\n", config.cardtype);
 	    break;
@@ -458,9 +470,24 @@ config_init(int argc, char **argv)
 	}
     }
     if (config.X) {
+	if (!config.X_keycode) {
+	    extern void keyb_layout(int layout);
+	    keyb_layout(-1);
+	    c_printf("CONF: Forceing neutral Keyboard-layout, X-server will translate\n");
+	}
 	config.console_video = config.vga = config.graphics = 0;
     }
     check_for_env_autoexec_or_config();
+    if (under_root_login)  c_printf("CONF: running exclusively as ROOT:");
+    else {
+#ifdef RUN_AS_ROOT
+      c_printf("CONF: mostly running as ROOT:");
+#else
+      c_printf("CONF: mostly running as USER:");
+#endif
+    }
+    c_printf(" uid=%d (cached %d) gid=%d (cached %d)\n",
+        geteuid(), get_cur_euid(), getegid(), get_cur_egid());
 }
 
 
@@ -516,9 +543,9 @@ int parse_debugflags(const char *s, unsigned char flag)
     char            c;
 
 #ifdef X_SUPPORT
-    const char      allopts[] = "dRWDCvXkism#pgcwhIExMnPrS";
+    const char      allopts[] = "dRWDCvXkiTsm#pgcwhIExMnPrS";
 #else
-    const char      allopts[] = "dRWDCvkism#pgcwhIExMnPrS";
+    const char      allopts[] = "dRWDCvkiTsm#pgcwhIExMnPrS";
 #endif
 
     /*
@@ -567,6 +594,9 @@ int parse_debugflags(const char *s, unsigned char flag)
 	case 'i':		/* i/o instructions (in/out) */
 	    d.io = flag;
 	    break;
+	case 'T':		/* i/o port tracing */
+	    d.io_trace = flag;
+	    break;
 	case 's':		/* serial */
 	    d.serial = flag;
 	    break;
@@ -640,39 +670,47 @@ int parse_debugflags(const char *s, unsigned char flag)
 static void
 usage(void)
 {
-    fprintf(stdout, "$Header: /usr/src/dosemu0.60/init/RCS/config.c,v 1.6 1995/05/06 16:26:13 root Exp root $\n");
+    fprintf(stdout, "dosemu 0.66\n");
     fprintf(stdout, "usage: dos [-ABCckbVNtsgxKm234e] [-D flags] [-M SIZE] [-P FILE] [ -F File ] 2> dosdbg\n");
+    fprintf(stdout, "    -2,3,4,5 choose 286, 386, 486 or 586 CPU\n");
     fprintf(stdout, "    -A boot from first defined floppy disk (A)\n");
     fprintf(stdout, "    -B boot from second defined floppy disk (B) (#)\n");
+    fprintf(stdout, "    -b map BIOS into emulator RAM (%%)\n");
     fprintf(stdout, "    -C boot from first defined hard disk (C)\n");
     fprintf(stdout, "    -c use PC console video (!%%)\n");
-    fprintf(stdout, "    -k use PC console keyboard (!)\n");
+    fprintf(stdout, "    -d detach (?)\n");
 #ifdef X_SUPPORT
     fprintf(stdout, "    -X run in X Window (#)\n");
 /* seems no longer valid bo 18.7.95
-    fprintf(stdout, "    -X NAME use MDA direct and FIFO NAME for keyboard (only with x2dos!)\n");
-    fprintf(stdout, "    -Y NAME use FIFO NAME for mouse (only with x2dos!)\n");
+    fprintf(stdout, "    -Y NAME use MDA direct and FIFO NAME for keyboard (only with x2dos!)\n");
+    fprintf(stdout, "    -Z NAME use FIFO NAME for mouse (only with x2dos!)\n");
 */
-    fprintf(stdout, "    -D set debug-msg mask to flags (+-)(dRWDCvXkism#pgcwhIExMnPrS01)\n");
+    fprintf(stdout, "    -D set debug-msg mask to flags {+-}{0-9}{#CDEIMPRSWXcdghikmnprsvwx}\n");
 #else				/* X_SUPPORT */
-    fprintf(stdout, "    -D set debug-msg mask to flags (+-)(dRWDCvkism#pgcwhIExMnPrS01)\n");
+    fprintf(stdout, "    -D set debug-msg mask to flags {+-}{0-9}{#CDEIMPRSWcdghikmnprsvwx}\n");
 #endif				/* X_SUPPORT */
-    fprintf(stdout, "    -M set memory size to SIZE kilobytes (!)\n");
-    fprintf(stdout, "    -P copy debugging output to FILE\n");
-    fprintf(stdout, "    -b map BIOS into emulator RAM (%%)\n");
-    fprintf(stdout, "    -F use config-file File\n");
-    fprintf(stdout, "    -V use BIOS-VGA video modes (!#%%)\n");
-    fprintf(stdout, "    -N No boot of DOS\n");
-    fprintf(stdout, "    -t try new timer code (#)\n");
-    fprintf(stdout, "    -s try new screen size code (#)\n");
-    fprintf(stdout, "    -g enable graphics modes (!%%#)\n");
-    fprintf(stdout, "    -x SIZE enable SIZE K XMS RAM\n");
+    fprintf(stdout, "       #=defint  C=cdrom    D=dos    E=ems       I=ipc     M=dpmi\n");
+    fprintf(stdout, "       P=packet  R=diskread S=sound  W=diskwrite c=config  d=disk\n");
+    fprintf(stdout, "       g=general h=hardware i=i/o    k=keyb      m=mouse   n=ipxnet\n");
+    fprintf(stdout, "       p=printer r=pic      s=serial v=video     w=warning x=xms\n");
+    fprintf(stdout, "    -E STRING pass DOS command on command line\n");
     fprintf(stdout, "    -e SIZE enable SIZE K EMS RAM\n");
+    fprintf(stdout, "    -F use config-file File\n");
+    fprintf(stdout, "    -I insert config statements (on commandline)\n");
+    fprintf(stdout, "    -g enable graphics modes (!%%#)\n");
+    fprintf(stdout, "    -K Do int9 (!#)\n");
+    fprintf(stdout, "    -k use PC console keyboard (!)\n");
+    fprintf(stdout, "    -M set memory size to SIZE kilobytes (!)\n");
     fprintf(stdout, "    -m enable mouse support (!#)\n");
-    fprintf(stdout, "    -o FILE put debugmessages in file\n");
+    fprintf(stdout, "    -N No boot of DOS\n");
     fprintf(stdout, "    -O write debugmessages to stderr\n");
-    fprintf(stdout, "    -2,3,4,5 choose 286, 386, 486 or 586 CPU\n");
-    fprintf(stdout, "    -K Do int9 (!#)\n\n");
+    fprintf(stdout, "    -o FILE put debugmessages in file\n");
+    fprintf(stdout, "    -P copy debugging output to FILE\n");
+    fprintf(stdout, "    -T DIR set tmpdir\n");
+    fprintf(stdout, "    -t try new timer code (#)\n");
+    fprintf(stdout, "    -V use BIOS-VGA video modes (!#%%)\n");
+    fprintf(stdout, "    -v NUM force video card type\n");
+    fprintf(stdout, "    -x SIZE enable SIZE K XMS RAM\n");
     fprintf(stdout, "    (!) BE CAREFUL! READ THE DOCS FIRST!\n");
     fprintf(stdout, "    (%%) require dos be run as root (i.e. suid)\n");
     fprintf(stdout, "    (#) options do not fully work yet\n");
