@@ -1,6 +1,3 @@
-/* 
- * $Id: init.c,v 1.4 1995/05/06 16:26:13 root Exp root $
- */
 #include <stdio.h>
 #include <string.h>
 #include <termios.h>
@@ -12,6 +9,7 @@
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
+#include <errno.h>
 
 #ifdef __linux__
 #include <linux/config.h>
@@ -46,6 +44,11 @@
 #include "xms.h"
 #include "shared.h"
 #include "iodev.h"
+#include "priv.h"
+
+#ifdef NEW_KBD_CODE
+#include "keyb_clients.h"
+#endif
 
 #ifdef USING_NET
 extern void pkt_check_receive_quick(void);
@@ -137,9 +140,9 @@ void stdio_init(void)
 void tmpdir_init(void)
 {
   /* create tmpdir */
-  priv_off();
+  enter_priv_off();
   mkdir(tmpdir, S_IREAD | S_IWRITE | S_IEXEC);
-  priv_default();
+  leave_priv_setting();
 }
 
 /*
@@ -174,12 +177,13 @@ void time_setting_init(void)
 void timer_interrupt_init(void)
 {
   struct itimerval itv;
+  int delta = (config.update / TIMER_DIVISOR);
 
   itv.it_interval.tv_sec = 0;
-  itv.it_interval.tv_usec = UPDATE / TIMER_DIVISOR;
+  itv.it_interval.tv_usec = delta;
   itv.it_value.tv_sec = 0;
-  itv.it_value.tv_usec = UPDATE / TIMER_DIVISOR;
-  k_printf("Used %d for updating timers\n", UPDATE / TIMER_DIVISOR);
+  itv.it_value.tv_usec = delta;
+  k_printf("Used %d for updating timers\n", delta);
   setitimer(TIMER_TIME, &itv, NULL);
 }
 
@@ -193,7 +197,7 @@ void timer_interrupt_init(void)
  */
 void hardware_setup(void)
 {
-
+  extern void  do_irq1(void);
   /* PIC init */
   pic_seti(PIC_IRQ0, timer_int_engine, 0);  /* do_irq0 in pic.c */
   pic_unmaski(PIC_IRQ0);
@@ -220,7 +224,7 @@ void hardware_setup(void)
   /* DMA Init */
   /* dma_init(); - Currently in dev_list */
 
-  g_printf("Hardware initialized\n");
+  g_printf("PIC,mouse,IPX,DMA initialized\n");
 }
 
 /*
@@ -372,6 +376,7 @@ static inline void bios_mem_setup(void)
   bios_configuration = configuration;
   bios_memory_size   = config.mem_size;	/* size of memory */
 
+#ifndef NEW_KBD_CODE
   /* The default 16-word BIOS key buffer starts at 0x41e */
 #if 0
   KBD_Head =			/* key buf start ofs */
@@ -392,6 +397,7 @@ static inline void bios_mem_setup(void)
 
   *OUTB_ADD = 1;                /* Set OUTB_ADD to 1 */
   *LASTSCAN_ADD = 1;
+#endif
 }
 
 /* 
@@ -469,7 +475,9 @@ void memory_init(void)
     ems_init();                /* initialize ems */
     xms_init();                /* initialize xms */
     shared_memory_init();
+#ifndef NEW_KBD_CODE
     shared_keyboard_init();
+#endif
   }
   first_call = 0;
 #if 0
@@ -492,11 +500,27 @@ void memory_init(void)
 void device_init(void)
 {
   port_init();
+
+#ifdef NEW_KBD_CODE
+  /* check whether we are running on the console */
+  check_console();
+
+  if (!keyb_server_init()) {
+    error("ERROR: can't init keyboard server\n");
+    leavedos(19);
+  }
+  if (!keyb_client_init()) {
+    error("ERROR: can't open keyboard client\n");
+    leavedos(19);
+  }
+#else
   if (keyboard_init() != 0) {
     error("ERROR: can't open keyboard\n");
     leavedos(19);
   }
   keyboard_flags_init();
+#endif
+   
   if (!config.vga)
     config.allowvideoportaccess = 0;
  
@@ -543,6 +567,7 @@ void low_mem_init(void)
  */
 void version_init(void) {
   struct new_utsname unames;
+  char version[80];
 
   uname((struct utsname *)&unames);
   warn("DOSEMU-%s is coming up on %s version %s\n", VERSTR, unames.sysname, unames.release);
@@ -552,13 +577,29 @@ void version_init(void) {
 #ifdef __NetBSD__
   warn("Built for %d\n", NETBSD_VERSION);
 #endif
+
+#ifdef __linux__
+  strcpy(version,unames.release);
+  running_kversion = atoi(strtok(version,".")) *1000000;
+  running_kversion += atoi(strtok(NULL,".")) *1000;
+  running_kversion += atoi(strtok(NULL,"."));
+  use_sigio=FASYNC;
+#endif
+  
+
+#if 0 /* hmm, the below _allway_ has been hit in the past,
+       * because: unames.release[2] > 1 always is true
+       * (unames.release is a string like "2.0.28") --Hans
+       */
   if (unames.release[0] > 0 ) {
     if ((unames.release[2] == 1  && unames.release[3] > 1 ) ||
          unames.release[2] > 1 ) {
       use_sigio=FASYNC;
     }
   }
+#endif
 
+#ifndef NEW_KBD_CODE
   /* Next Check input */
   if (isatty(STDIN_FILENO)) {
     k_printf("STDIN is tty\n");
@@ -567,7 +608,8 @@ void version_init(void) {
     k_printf("STDIN not a tty\n");
     config.kbd_tty = 1;
   }
-
+#endif
+   
 }
 
 
