@@ -13,20 +13,6 @@
  * DANG_END_MODULE
  *
  *
- * DANG_BEGIN_CHANGELOG
- *
- * $$
- * $Source: /usr/src/dosemu0.60/keyboard/RCS/keyboard-server.c,v $
- * $Revision: 1.2 $
- * $State: Exp $
- * $Log: keyboard-server.c,v $
- * Revision 1.2  1995/05/06  16:27:36  root
- * Prep for 0.60.2.
- *
- * Revision 1.1  1995/04/08  22:34:30  root
- * Initial revision
- *
- * DANG_END_CHANGELOG
  */
 
 #include <unistd.h>
@@ -36,8 +22,10 @@
 #ifdef __linux__
 #include <linux/vt.h>
 #include <linux/kd.h>
+#if ! (__GLIBC__ > 1)
 #include <linux/time.h>
 #endif
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #ifdef __NetBSD__
@@ -591,11 +579,15 @@ set_leds_area ()
   }
 }
 
+static char accent = 0;
+static short accent_sc = 0;
+static short very_next_scancode = 0;
+
 static void
 do_self (unsigned int sc)
 {
   unsigned char ch;
-  static char accent = 0;
+  int out_accent;
   int i;
 
   if (kbd_flag (KF_ALT))
@@ -637,6 +629,7 @@ do_self (unsigned int sc)
 	if (accent != ch) {
 	   k_printf("KBD: dead key accent %d\n", ch);
 	   accent=ch;
+	   accent_sc = sc;
 	   return;
 	}
      }
@@ -644,6 +637,10 @@ do_self (unsigned int sc)
 
 
   if (accent) {   /* translate dead keys */
+     out_accent = 0;
+     if(ch == ' ')
+       /* dead-key + space becomes lone accent */
+       ch = accent;
      for (i = 0; dos850_dead_map[i].d_key != 0; i++) {
 	if (accent == dos850_dead_map[i].d_key &&
 	    dos850_dead_map[i].in_key == ch) {
@@ -654,7 +651,18 @@ do_self (unsigned int sc)
 	    sc = 0; /* keyb.exe uses 0 for dead keys */
             break;
 	}
+	if (accent == dos850_dead_map[i].d_key &&
+	    dos850_dead_map[i].in_key == accent) {
+
+	    k_printf("KBD: map accent %d/key %d to %d\n", accent, ch,
+		     dos850_dead_map[i].out_key);
+	    out_accent = dos850_dead_map[i].out_key;
+	}
      }
+     /* For an unknown 'dead key' sequence, generate accent+following character
+      * as two characters, as real DOS does */
+     if(sc && out_accent)
+       very_next_scancode = (accent_sc << 8) | out_accent;
      accent = 0;
   }
   if (kbd_flag (KF_CTRL) || kbd_flag (KF_CAPSLOCK))
@@ -672,7 +680,10 @@ do_self (unsigned int sc)
 static void
 spacebar (unsigned int sc)
 {
-  put_queue (0x3920);
+  if(accent)
+    do_self(sc);
+  else
+    put_queue (0x3920);
 }
 
 /* 0x47-0x53, indexed by sc-0x47 , goes like:
@@ -1018,7 +1029,7 @@ set_keyboard_bios (void)
   else
     next_scancode = convscanKey(next_scancode);
   keys_ready = 0;		/* flag character as read	*/
-  k_printf ("set keybaord bios next_scancode = 0x%04x\n", next_scancode);
+  k_printf ("set keyboard bios next_scancode = 0x%04x\n", next_scancode);
 }
 
 void
@@ -1031,6 +1042,16 @@ insert_into_keybuffer (void)
   else
     keepkey = 0;
 
+  if (very_next_scancode && keepkey)
+    {
+      k_printf ("KBD: putting key in buffer\n");
+      if (InsKeyboard (very_next_scancode))
+	dump_kbuffer ();
+      else
+	error ("ERROR: InsKeyboard could not put key into buffer!\n");
+      very_next_scancode=0;
+    }
+
   if (next_scancode && keepkey)
     {
       k_printf ("KBD: putting key in buffer\n");
