@@ -8,6 +8,7 @@
  * Unix mounted drives that permissions exist for.
  *
  * DANG_END_MODULE
+ * DANG_FIXTHIS We probably should use llseek here for file > 2 GBytes
  *
  * Copyright (c) 1991 Carnegie Mellon University
  * All Rights Reserved.
@@ -340,6 +341,9 @@ struct direct *dos_readdir(DIR *);
 
 #endif
 
+void build_ufs_path(char *ufs, char *path);
+char *is_reserved_msdos (char *s);
+
 /* Try and work out if the current command is for any of my drives */
 static int
 select_drive(state)
@@ -515,20 +519,6 @@ select_drive(state)
 	bs_pos = i;
     }
     free(nametmp);
-
-/*
- * DANG_BEGIN_REMARK
- * 	This code is to handle special files that should be reflected as
- *	devices. At this time, only PRN has been tested. Others like LPT[0-x]
- *	etc... should be added.
- * DANG_END_FUNCTION
- */
-    if (strcmp(fpath + bs_pos + 1, "prn") == 0) {
-	Debug0((dbg_fd, "Special File, so name changed to '%s'\n", fpath + bs_pos + 1));
-        strcpy(name, fpath + bs_pos + 1);
-	return(0);
-    }
-
   }
 
   /* for find next we will check the drive letter in the
@@ -620,8 +610,8 @@ get_disk_space(char *cwd, int *free, int *total)
   struct statfs fsbuf;
 
   if (statfs(cwd, &fsbuf) >= 0) {
-    *free = fsbuf.f_bsize * fsbuf.f_bavail / 512;
-    *total = fsbuf.f_bsize * fsbuf.f_blocks / 512;
+    *free = (fsbuf.f_bsize / 512) * fsbuf.f_bavail;
+    *total = (fsbuf.f_bsize / 512) * fsbuf.f_blocks;
     return (1);
   }
   else
@@ -794,19 +784,20 @@ mfs_redirector(void)
   int dos_fs_redirect();
   int ret;
 
-  if (!priv_off())
-    return 0;
-
   PS(MFS);
+  if (!enter_priv_off()) {
+    leave_priv_setting();
+    return 0;
+  }
+
   ret = dos_fs_redirect(&REGS);
+  leave_priv_setting();
   PE(MFS);
 
   Debug0((dbg_fd, "Finished dos_fs_redirect\n"));
 
   finds_in_progress[current_drive] = find_in_progress;
 
-  priv_default();
-
   switch (ret) {
   case FALSE:
     REG(eflags) |= CF;
@@ -829,11 +820,12 @@ mfs_inte6(void)
   boolean_t dos_fs_dev();
   boolean_t result;
 
-  if (!priv_off())
+  if (!enter_priv_off()) {
+    leave_priv_setting();
     return 0;
-
+  }
   result = dos_fs_dev(&REGS);
-  priv_default();
+  leave_priv_setting();
   return (result);
 }
 
@@ -1684,12 +1676,33 @@ build_ufs_path(ufs, path)
      char *ufs;
      char *path;
 {
-  strcpy(ufs, dos_root);
+  char *p, *s;
 
   Debug0((dbg_fd, "dos_fs: build_ufs_path for DOS path '%s'\n", path));
 
-  /* Skip over leading <drive>:\ */
-  path += cds_rootlen(cds);
+ if ((s=is_reserved_msdos(path))) {
+  Debug0((dbg_fd,"MFS: Special File, so name changed to '%s'\n", s));
+  strcpy (ufs, s);
+ }
+ else {
+  /* Set ufs to dos_roots[current_drive] */
+  strcpy(ufs, dos_root);
+  /* Skip over leading <drive>:\ in the path */
+  if (path[1]==':')
+      path += cds_rootlen(cds);
+
+  /* remove spaces */
+  s = path; p = NULL;
+  while (*s && (*s==' ')) *s++ = '_';
+  while (*s && (*s!='.')) {
+  	if ((*s==' ') && (p==NULL)) p=s;
+  	s++;
+  }
+  if (p) {	/* space found */
+  	if (*s=='.') { while (*s && (*s!=' ')) *p++=*s++; }
+  	*p = 0;
+  }
+  Debug0((dbg_fd,"MFS: dos_gen: ufs '%s', path '%s', l=%d\n", ufs, path, dos_root_len));
 
   path_to_ufs(ufs + dos_root_len, path, 0);
 
@@ -1701,7 +1714,7 @@ build_ufs_path(ufs, path)
     else
       path++;
   }
-
+ }
   Debug0((dbg_fd, "dos_fs: build_ufs_path result is '%s'\n", ufs));
 }
 
@@ -1832,10 +1845,12 @@ _find_file(char *fpath, struct stat * st)
 	  strcat(slash1+1,remainder);
 	return (FALSE);
       }
-      *slash1 = '/';
-      if (slash2)
+      else {
+        *slash1 = '/';
+        if (slash2)
 	  strcat(slash1+1,remainder);
-      slash1 = strchr(slash1+1,'/');
+        slash1 = strchr(slash1+1,'/');
+      }
     }
   }
 
@@ -2229,6 +2244,7 @@ RedirectDevice(state_t * state)
   /* first, see if this is our resource to be redirected */
   resourceName = (u_char *) Addr(state, es, edi);
   deviceName = (u_char *) Addr(state, ds, esi);
+  path[0] = 0;
 
   Debug0((dbg_fd, "RedirectDevice %s to %s\n", deviceName, resourceName));
   if (strncmp(resourceName, LINUX_RESOURCE,
@@ -2627,28 +2643,33 @@ dos_fs_redirect(state)
   case GET_DISK_SPACE:
     {				/* 0x0c */
 #ifdef USE_DF_AND_AFS_STUFF
-      int free, tot;
+      unsigned int free, tot;
 
       Debug0((dbg_fd, "Get Disk Space\n"));
       build_ufs_path(fpath, cds_current_path(cds));
 
       if (find_file(fpath, &st)) {
 	if (get_disk_space(fpath, &free, &tot)) {
+	  /* return unit = 512-byte blocks @ 1 spc, std for floppy */
 	  int spc = 1;
 	  int bps = 512;
-	  int tmpf, tmpt;
 
-	  if ((tot > 256 * 256) || (free > 256 * 256)) {
-	    tmpf = free * bps * spc;
-	    tmpt = tot * bps * spc;
-
-	    spc = 8;
-	    bps = 1024;
-
-	    free = (tmpf / bps) / spc;
-	    tot = (tmpt / bps) / spc;
+	  while ((spc<32) && ((tot > 65535) || (free > 65535))) {
+	    /* we're on an HD here! */
+	    if (bps==512) {
+	      bps = 1024;	/* try 1024-byte sectors first */
+	    }
+	    else
+	      spc *= 2;		/* try bigger clusters, up to 32 */
+	    free /= 2;
+	    tot  /= 2;
 	  }
+	  /* report no more than 32*1024*64K = 2G, even if some
+	     DOS version 7 can see more */
+	  if (tot>65535) tot=65535;
+	  if (free>65535) free=65535;
 
+	  /* Ralf Brown says: AH=media ID byte - can we let it at 0 here? */
 	  SETWORD(&(state->eax), spc);
 	  SETWORD(&(state->edx), free);
 	  SETWORD(&(state->ecx), bps);
@@ -2955,6 +2976,7 @@ dos_fs_redirect(state)
 
     /* make it a byte - we thus ignore the new bit */
     attr &= 0xFF;
+    if (attr & DIRECTORY) return(REDIRECT);
 
     Debug0((dbg_fd, "Create truncate file %s attr=%x\n", filename1, attr));
 
