@@ -39,7 +39,6 @@
 #include "mouse.h"
 #include "serial.h"
 #include "xms.h"
-#include "keyboard.h"
 #include "timers.h"
 #include "cmos.h"
 #include "memory.h"
@@ -52,6 +51,8 @@
 #include "timers.h"
 
 #include "video.h"
+#include "priv.h"
+
 #if X_GRAPHICS
 #include "vgaemu.h" /* root@zaphod */
 #endif
@@ -70,8 +71,41 @@
 #include "sound.h"
 #endif
 
+#ifdef NEW_KBD_CODE
+#include "keyb_server.h"
+#endif
+#include "keyboard.h"
+
 #include "dma.h"
 
+extern Bit8u spkr_io_read(Bit32u port);
+extern void spkr_io_write(Bit32u port, Bit8u value);
+
+/* ====================================================================== */
+
+static long nyb2bin[16] = {
+	0x30303030, 0x31303030, 0x30313030, 0x31313030,
+	0x30303130, 0x31303130, 0x30313130, 0x31313130,
+	0x30303031, 0x31303031, 0x30313031, 0x31313031,
+	0x30303131, 0x31303131, 0x30313131, 0x31313131
+};
+
+static char *
+p2bin (unsigned char c)
+{
+  static char s[16] = "   [00000000]";
+
+  ((long *)s)[1]=nyb2bin[(c>>4)&15];
+  ((long *)s)[2]=nyb2bin[c&15];
+  
+  return s+3;
+}
+
+#ifndef USE_MRP_JOYSTICK
+static int fake201 = 50;	/* for diagnostic loops, do not run games
+				   with THIS on */
+#endif
+
 /* ====================================================================== */
 /*  */
 /* inb,inw,ind,outb,outw,outd @@@  32768 MOVED_CODE_BEGIN @@@ 01/23/96, ./src/arch/linux/async/sigsegv.c --> src/emu-i386/ports.c  */
@@ -83,9 +117,22 @@
 */ 
 #define PORT_DEBUG 3
 
-extern void set_leds(void);
-/* FIXME -- move to common header */
-extern int s3_8514_base;
+
+/*                              3b0         3b4         3b8         3bc                     */
+const unsigned char ATIports[]={ 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, /* 3b0-3bf */
+                                 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, /* 3c0-3cf */
+                                 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0  /* 3d0-3df */
+                               };
+
+int isATIport(int port)
+{
+  return (    (port & 0x3ff) == 0x102
+          ||  (port & 0x3fe) == 0x1ce
+          ||  (port & 0x3fc) == 0x2ec
+          || ((port & 0x3ff) >= 0x3b0 && (port & 0x3ff) < 0x3e0 && ATIports[(port & 0x3ff)-0x3b0])
+          ||  port == 0x46e8
+         );
+}
 
 /*
  * DANG_BEGIN_FUNCTION inb
@@ -127,6 +174,7 @@ inb(unsigned int port)
 #if X_GRAPHICS
   if ( (config.X) &&
             ( ((port>=0x3c0) && (port<=0x3c1)) || /* root@zaphod attr ctrl */
+              ((port>=0x3c4) && (port<=0x3c5)) || /* erik@zaphod sequencer */
               ((port>=0x3c6) && (port<=0x3c9)) || /* root@zaphod */
               ((port>=0x3D0) && (port<=0x3DD)) ) )
     {
@@ -156,13 +204,23 @@ inb(unsigned int port)
     }
   }
 #if 1
-  else if (config.chipset && port > 0x3b3 && port < 0x3df && config.mapped_bios)
+  if (config.chipset && (port > 0x3b3) && (port < 0x3df) && config.mapped_bios)
     r = (video_port_in((u_int)port));
-  else if ((config.chipset == S3) && ((port & 0x03fe) == s3_8514_base) && (port & 0xfc00)) {
+  else if (v_8514_base && ((port & 0x03fe) == v_8514_base) && (port & 0xfc00)) {
+    enter_priv_on();
     iopl(3);
     r = port_in((u_int)port) & 0xff;
     iopl(0);
-    v_printf("S3 inb [0x%04x] = 0x%02x\n", port, r);
+    leave_priv_setting();
+    v_printf("8514 inb [0x%04x] = 0x%02x\n", port, r);
+  }
+  else if ((config.chipset == ATI) && isATIport(port) && (port & 0xfc00)) {
+    enter_priv_on();
+    iopl(3);
+    r = port_in(port) & 0xff;
+    iopl(0);
+    leave_priv_setting();
+    v_printf("ATI inb [0x%04x] = 0x%02x\n", port, r);
   }
 #endif
   else switch ((u_int)port) {
@@ -217,9 +275,11 @@ inb(unsigned int port)
     return r;
 
   case 0xa79:
+    enter_priv_on();
     iopl (3);
     r = port_in (port);
     iopl (0);
+    leave_priv_setting();
     break;
 #endif
   default:
@@ -238,6 +298,11 @@ inb(unsigned int port)
     }
     /* It seems that we might need 388, but this is write-only, at least in the
        older chip... */
+    /* MPU401 */
+    if ((port & ~1) == config.mpu401_base) {
+	r=mpu401_io_read(port);
+	return r;
+    }
 #endif /* USE_SBEMU */
 
     /* DMA I/O */
@@ -247,9 +312,11 @@ inb(unsigned int port)
 
     /* The diamond bug */
     else if (config.chipset == DIAMOND && (port >= 0x23c0) && (port <= 0x23cf)) {
+      enter_priv_on();
       iopl(3);
       r = port_in(port);
       iopl(0);
+      leave_priv_setting();
       i_printf(" Diamond inb [0x%x] = 0x%x\n", port, r);
     }
     else {
@@ -275,13 +342,26 @@ inb(unsigned int port)
 unsigned int
 inw(int port)
 {
-  if ((config.chipset == S3) && ((port & 0x03ff) == s3_8514_base) && (port & 0xfc00)) {
+  if (v_8514_base && ((port & 0x03fd) == v_8514_base) && (port & 0xfc00)) {
     int value;
 
+    enter_priv_on();
     iopl(3);
     value = port_in_w(port) & 0xffff;
     iopl(0);
-    v_printf("S3 inw [0x%04x] = 0x%04x\n", port, value);
+    leave_priv_setting();
+    v_printf("8514 inw [0x%04x] = 0x%04x\n", port, value);
+    return value;
+  }
+  else if ((config.chipset == ATI) && isATIport(port) && (port & 0xfc00)) {
+    int value;
+
+    enter_priv_on();
+    iopl(3);
+    value = port_in_w(port) & 0xffff;
+    iopl(0);
+    leave_priv_setting();
+    v_printf("ATI inw [0x%04x] = 0x%04x\n", port, value);
     return value;
   }
   return( read_port_w(port) );
@@ -377,6 +457,7 @@ outb(unsigned int port, unsigned int byte)
 #if X_GRAPHICS
     if ( (config.X) &&
          ( ((port>=0x3c0) && (port<=0x3c1)) || /* root@zaphod attr ctrl */
+           ((port>=0x3c4) && (port<=0x3c5)) || /* erik@zaphod sequencer */
            ((port>=0x3c6) && (port<=0x3c9)) || /* root@zaphod */
            ((port>=0x3D0) && (port<=0x3DD)) ) )
       {
@@ -415,23 +496,36 @@ outb(unsigned int port, unsigned int byte)
     }
   }
 
-#if 1
-  if (port > 0x3b3 && port < 0x3df && config.chipset && config.mapped_bios)
+  if (port > 0x3b3 && port < 0x3df && config.chipset && config.mapped_bios) {
     video_port_out(byte, port);
-  if ((config.chipset == S3) && ((port & 0x03fe) == s3_8514_base) && (port & 0xfc00)) {
+    return;
+  }
+  if (v_8514_base && ((port & 0x03fe) == v_8514_base) && (port & 0xfc00)) {
+    enter_priv_on();
     iopl(3);
     port_out(byte, port);
     iopl(0);
-    v_printf("S3 outb [0x%04x] = 0x%02x\n", port, byte);
+    leave_priv_setting();
+    v_printf("8514 outb [0x%04x] = 0x%02x\n", port, byte);
+    return;
+  }
+  if ((config.chipset == ATI) && isATIport(port) && (port & 0xfc00)) {
+    enter_priv_on();
+    iopl(3);
+    port_out(byte, port);
+    iopl(0);
+    leave_priv_setting();
+    v_printf("ATI outb [0x%04x] = 0x%02x\n", port, byte);
     return;
   }
-#endif
 
   /* The diamond bug */
   if (config.chipset == DIAMOND && (port >= 0x23c0) && (port <= 0x23cf)) {
+    enter_priv_on();
     iopl(3);
     port_out(byte, port);
     iopl(0);
+    leave_priv_setting();
     i_printf(" Diamond outb [0x%x] = 0x%x\n", port, byte);
     return;
   }
@@ -485,13 +579,19 @@ outb(unsigned int port, unsigned int byte)
     break;
 
   case 0xa79:
+    enter_priv_on();
     iopl (3);
     port_out (byte, port);
     iopl (0);
+    leave_priv_setting();
   break;
 #endif
 
   default:
+    /* DMA I/O */
+    if (((port & ~15) == 0) || ((port & ~15) == 0x80) || ((port & ~31) == 0xC0)) { 
+      dma_io_write(port, byte);
+    }
     /* SERIAL PORT I/O.  Avoids port==0 for safety.  */
     /* The base serial port must be a multiple of 8. */
     for (tmp = 0; tmp < config.num_ser; tmp++) {
@@ -511,29 +611,39 @@ outb(unsigned int port, unsigned int byte)
       adlib_io_write(port, byte);
       break;
     }
+    /* MPU401 */
+    if ((port & ~1) == config.mpu401_base) {
+      mpu401_io_write(port, byte);
+      break;
+    }
 #endif /* USE_SBEMU */
 
-    /* DMA I/O */
-    if (((port & ~15) == 0) || ((port & ~15) == 0x80) || ((port & ~31) == 0xC0)) {
-      dma_io_write(port, byte);
-    }
-    else {
     i_printf("default outb [0x%x] 0x%02x\n", port, byte);
-      h_printf("write port 0x%x denied value %02x", port, (byte & 0xff));
+    h_printf("write port 0x%x denied value %02x", port, (byte & 0xff));
     h_printf(" because not in access list\n");
   }
-  }
   lastport = port;
 }
 
 void
 outw(unsigned int port, unsigned int value)
 {
-  if ((config.chipset == S3) && ((port & 0x03ff) == s3_8514_base) && (port & 0xfc00)) {
+  if (v_8514_base && ((port & 0x03fd) == v_8514_base) && (port & 0xfc00)) {
+    enter_priv_on();
     iopl(3);
     port_out_w(value, port);
     iopl(0);
-    v_printf("S3 outw [0x%04x] = 0x%04x\n", port, value);
+    leave_priv_setting();
+    v_printf("8514 outw [0x%04x] = 0x%04x\n", port, value);
+    return;
+  }
+  if ((config.chipset == ATI) && isATIport(port) && (port & 0xfc00)) {
+    enter_priv_on();
+    iopl(3);
+    port_out(value, port);
+    iopl(0);
+    leave_priv_setting();
+    v_printf("ATI outb [0x%04x] = 0x%02x\n", port, value);
     return;
   }
   if(!write_port_w(value,port) ) {
@@ -565,9 +675,9 @@ set_ioperm(int start, int size, int flag)
     if (!i_am_root)
 	return -1;		/* don't bother */
 
-    priv_on();
+    enter_priv_on();
     tmp = ioperm(start, size, flag);
-    priv_default();
+    leave_priv_setting();
 
     return tmp;
 }
