@@ -17,20 +17,9 @@
  * vc.c - for the Linux DOS emulator
  *  Robert Sanders, gt8134b@prism.gatech.edu
  *
- * DANG_BEGIN_CHANGELOG
- *
- * $Date: 1995/04/08 22:30:27 $
- * $Source: /usr/src/dosemu0.60/video/RCS/vc.c,v $
- * $Revision: 1.14 $
- * $State: Exp $
- *
- * DANG_END_CHANGELOG
  */
 
 
-/* why ?? */
-#define VGA_SAVE_PAGE0
-
 #include <stdio.h>
 #include <unistd.h>
 #include <sys/ioctl.h>
@@ -78,6 +67,7 @@ extern int munmap __P ((caddr_t __addr, size_t __len));
 #include "s3.h"
 #include "trident.h"
 #include "et4000.h"
+#include "priv.h"
 
 extern void child_close_mouse ();
 extern void child_open_mouse ();
@@ -86,6 +76,7 @@ extern inline void console_update_cursor (int, int, int, int);
 static void set_dos_video ();
 void put_video_ram ();
 
+static struct debug_flags save_debug_flags;
 
 extern int
   dosemu_sigaction (int sig, struct sigaction *, struct sigaction *);
@@ -125,6 +116,54 @@ forbid_switch (void)
   scr_state.vt_allow = 0;
 }
 
+/* check whether we are running on the console; initialise
+ * config.console, console_fd and scr_state.console_no
+ */
+void check_console() {
+
+#ifdef __NetBSD__
+    struct screeninfo info;
+    int result;
+
+    config.console=0;
+    console_fd = STDIN_FILENO;     /* not STDOUT_FILENO */
+
+    if (ioctl(console_fd, VGAGETSCREEN, &info)==0) {
+	/* it's zero-based, we use 1-based */
+	scr_state.console_no = info.screen_no + 1;
+        config.console=1;
+    }
+#endif
+
+#ifdef __linux__
+    struct stat chkbuf;
+    int major, minor;
+
+    config.console=0;
+    console_fd = STDIN_FILENO;
+
+    fstat(console_fd, &chkbuf);
+    major = chkbuf.st_rdev >> 8;
+    minor = chkbuf.st_rdev & 0xff;
+
+    /* console major num is 4, minor 64 is the first serial line */
+    if ((major == 4) && (minor < 64)) {
+       scr_state.console_no = minor;
+       config.console=1;
+    }
+#endif
+}
+
+void
+vt_activate(int con_num)
+{
+    if (in_ioctl) {
+	k_printf("KBD: can't ioctl for activate, in a signal handler\n");
+/*	do_ioctl(console_fd, VT_ACTIVATE, con_num); */
+    } else
+	do_ioctl(console_fd, VT_ACTIVATE, con_num);
+}
+
 void
 allow_switch (void)
 {
@@ -155,7 +194,9 @@ parent_open_mouse (void)
 {
   if (mice->intdrv)
     {
+      enter_priv_on(); /* The mouse may not be a resource everyone can open. */
       mice->fd = DOS_SYSCALL (open (mice->dev, O_RDWR | O_NONBLOCK));
+      leave_priv_setting();
       if (mice->fd > 0)
 	add_to_io_select(mice->fd, mice->add_to_io_select);
     }
@@ -182,13 +223,14 @@ acquire_vt (int sig, struct sigcontext_struct context)
 {
   dos_has_vt = 1;
 
+  d = save_debug_flags;
   v_printf ("VID: Acquiring VC\n");
   forbid_switch ();
 #ifdef __NetBSD__
   if (config.console_keyb)
       set_raw_mode();
 #endif
-  if (ioctl (kbd_fd, VT_RELDISP, VT_ACKACQ))	/* switch acknowledged */
+  if (ioctl (console_fd, VT_RELDISP, VT_ACKACQ))	/* switch acknowledged */
     v_printf ("VT_RELDISP failed (or was queued)!\n");
   allow_switch ();
   SIGNAL_save (SIGACQUIRE_call);
@@ -260,6 +302,7 @@ set_linux_video (void)
 static void
 SIGRELEASE_call (void)
 {
+  save_debug_flags = d;
 
   if (scr_state.current == 1)
     {
@@ -306,14 +349,19 @@ SIGRELEASE_call (void)
     }
 
   scr_state.current = 0;	/* our console is no longer current */
-  if (do_ioctl (kbd_fd, VT_RELDISP, 1))	/* switch ok by me */
+  if (do_ioctl (console_fd, VT_RELDISP, 1))	/* switch ok by me */
     v_printf ("VT_RELDISP failed!\n");
+  else {
+   /* we don't want any debug activity while VC is not active */
+    flush_log();
+    memset(&d, 0, sizeof(struct debug_flags));
+  }
 }
 
 int
 wait_vc_active (void)
 {
-  if (ioctl (kbd_fd, VT_WAITACTIVE, scr_state.console_no) < 0)
+  if (ioctl (console_fd, VT_WAITACTIVE, scr_state.console_no) < 0)
     {
       if (errno != EINTR)
 		{
@@ -558,7 +606,7 @@ set_process_control (void)
   NEWSETQSIG (SIG_RELEASE, release_vt);
   NEWSETQSIG (SIG_ACQUIRE, acquire_vt);
 
-  if (do_ioctl (kbd_fd, VT_SETMODE, (int) &vt_mode))
+  if (do_ioctl (console_fd, VT_SETMODE, (int) &vt_mode))
     v_printf ("initial VT_SETMODE failed!\n");
   v_printf ("VID: Set process control\n");
 }
@@ -569,7 +617,9 @@ clear_process_control (void)
   struct vt_mode vt_mode;
 
   vt_mode.mode = VT_AUTO;
-  ioctl (kbd_fd, VT_SETMODE, (int) &vt_mode);
+  enter_priv_on();
+  ioctl (console_fd, VT_SETMODE, (int) &vt_mode);
+  leave_priv_setting();
   signal (SIG_RELEASE, SIG_IGN);
   signal (SIG_ACQUIRE, SIG_IGN);
 }
@@ -596,9 +646,9 @@ open_kmem (void)
 
   if (mem_fd != -1)
     return;
-  priv_on();
+  enter_priv_on();
   mem_fd = open("/dev/mem", O_RDWR);
-  priv_default();
+  leave_priv_setting();
   if (mem_fd < 0)
     {
       error ("ERROR: can't open /dev/mem: errno=%d, %s \n",
@@ -631,7 +681,7 @@ vc_active (void)
   struct vt_stat vtstat;
 
   g_printf ("VC_ACTIVE!\n");
-  ioctl (kbd_fd, VT_GETSTATE, &vtstat);
+  ioctl (console_fd, VT_GETSTATE, &vtstat);
   g_printf ("VC_ACTIVE: ours: %d, active: %d\n", scr_state.console_no, vtstat.v_active);
   return ((vtstat.v_active == scr_state.console_no));
 #endif
@@ -640,7 +690,7 @@ vc_active (void)
   int active;
 
   g_printf ("VC_ACTIVE!\n");
-  ioctl (kbd_fd, VT_GETACTIVE, &active);
+  ioctl (console_fd, VT_GETACTIVE, &active);
   g_printf ("VC_ACTIVE: ours: %d, active: %d\n", scr_state.console_no, active);
   return ((active == scr_state.console_no));
 #endif
@@ -679,9 +729,15 @@ get_perm (void)
 	  v_printf ("VGA: can't get I/O permissions \n");
 	  exit (-1);
 	}
-      if (config.chipset == S3 && (set_ioperm (0x102, 1, 1) || set_ioperm (0x2ea, 4, 1)))
+      if (((config.chipset==S3) || (config.chipset==CIRRUS)) &&
+		(set_ioperm (0x102, 1, 1) || set_ioperm (0x2ea, 4, 1)))
 	{
-	  v_printf ("S3: can't get I/O permissions \n");
+	  v_printf ("S3/CIRRUS: can't get I/O permissions \n");
+	  exit (-1);
+	}
+      if (config.chipset == ATI && (set_ioperm (0x102, 1, 1) || set_ioperm (0x1ce, 2, 1) || set_ioperm (0x2ec, 4, 1)))
+	{
+	  v_printf ("ATI: can't get I/O permissions \n");
 	  exit (-1);
 	}
       /* color or monochrome text emulation? */
@@ -739,9 +795,16 @@ release_perm (void)
 	      v_printf ("VGA: can't release I/O permissions \n");
 	      leavedos (-1);
 	    }
-	  if (config.chipset == S3 && (set_ioperm (0x102, 1, 0) || set_ioperm (0x2ea, 4, 0)))
+	  if (((config.chipset==S3) || (config.chipset==CIRRUS)) && 
+		(set_ioperm (0x102, 1, 0) || set_ioperm (0x2ea, 4, 0)))
 	    {
-	      v_printf ("S3: can't release I/O permissions\n");
+	      v_printf ("S3/CIRRUS: can't release I/O permissions\n");
+	      leavedos (-1);
+	    }
+	  if (config.chipset == ATI && 
+		(set_ioperm (0x102, 1, 0) || set_ioperm (0x1ce, 2, 0) || set_ioperm (0x2ec, 4, 0)))
+	    {
+	      v_printf ("ATI: can't release I/O permissions \n");
 	      leavedos (-1);
 	    }
 	}
