@@ -1,46 +1,6 @@
 /* dos emulator, Matthias Lautner 
  * Extensions by Robert Sanders, 1992-93
  *
- * $Date: 1995/05/06 16:28:14 $
- * $Source: /usr/src/dosemu0.60/include/RCS/emu.h,v $
- * $Revision: 2.27 $
- * $State: Exp $
- *
- * $Log: emu.h,v $
- * Revision 2.27  1995/05/06  16:28:14  root
- * Prep for 0.60.2.
- *
- * Revision 2.26  1995/04/08  22:35:19  root
- * Release dosemu0.60.0
- *
- * Revision 2.25  1995/02/25  22:38:41  root
- * *** empty log message ***
- *
- * Revision 2.24  1995/02/25  21:54:46  root
- * *** empty log message ***
- *
- * Revision 2.23  1995/02/05  16:54:16  root
- * Prep for Scotts patches.
- *
- * Revision 2.22  1995/01/14  15:31:55  root
- * New Year checkin.
- *
- * Revision 2.21  1994/11/13  00:40:45  root
- * Prep for Hans's latest.
- *
- * Revision 2.20  1994/11/03  11:43:26  root
- * Checkin Prior to Jochen's Latest.
- *
- * Revision 2.19  1994/10/14  17:58:38  root
- * Prep for pre53_27.tgz
- *
- * Revision 2.18  1994/10/03  00:24:25  root
- * Checkin prior to pre53_25.tgz
- *
- * Revision 2.17  1994/09/26  23:10:13  root
- * Prep for pre53_22.
- *
- *
  */
 
 #ifndef EMU_H
@@ -92,9 +52,21 @@ EXTERN unsigned int use_sigio INIT(0);
 EXTERN unsigned int not_use_sigio INIT(0);
 EXTERN int terminal_pipe;
 EXTERN int terminal_fd INIT(-1);
+EXTERN int running_kversion INIT(0);
 
 /* set to one if runing setuid as root */
+#if 1
+#define i_am_root 1  /* neadless to have it variable, it will be allways 1
+                      * If dosemu isn't suid root, it will terminate
+                      */
+#else
 EXTERN int i_am_root INIT(0);
+#endif
+
+/* set, if dosemu was started from a 'root login',
+ * unset, if dosemu was started from a 'user login' (running suid root)
+ */
+EXTERN int under_root_login INIT(0);
 
 EXTERN char *cstack[16384];
 
@@ -125,7 +97,11 @@ EXTERN char *cl,		/* clear screen */
 *ve;				/* cursor normal */
 
 /* the fd for the keyboard */ 
-EXTERN int kbd_fd INIT(-1);
+EXTERN int console_fd INIT(-1);
+#ifndef NEW_KBD_CODE
+  #define kbd_fd console_fd
+#endif
+
 /* the file descriptor for /dev/mem when mmap'ing the video mem */
 EXTERN int mem_fd INIT(-1);
 EXTERN int in_readkeyboard;
@@ -143,9 +119,7 @@ EXTERN int cursor_col;
 
 void dos_ctrlc(void), dos_ctrl_alt_del(void);
 int ext_fs(int, char *, char *, int);
-#if 0 /* Not used anymore */
-int outch(int c);
-#endif
+
 EXTERN void run_vm86(void);
 EXTERN void     vm86_GP_fault();
 
@@ -168,6 +142,7 @@ struct debug_flags {
    X,			/* X support         "X" */
    keyb,		/* keyboard          "k" */
    io,			/* port I/O          "i" */
+   io_trace,		/* port I/O tracing  "T" */
    serial,		/* serial            "s" */
    mouse,		/* mouse             "m" */
    defint,		/* default ints      "#" */
@@ -200,7 +175,7 @@ struct debug_flags {
 
 EXTERN void saytime(char *m_str);
 
-int ifprintf(unsigned int, const char *,...) FORMAT(printf, 2, 3);
+int log_printf(int, const char *,...) FORMAT(printf, 2, 3);
 
 void p_dos_str(char *,...) FORMAT(printf, 1, 2);
 
@@ -208,15 +183,20 @@ void p_dos_str(char *,...) FORMAT(printf, 1, 2);
  #define NO_DEBUGPRINT_AT_ALL
 #endif
 
+extern FILE *dbg_fd;
+
+#define ifprintf(flg,fmt,a...)	do{ if (flg) log_printf(flg,fmt,##a); }while(0)
+
 /* Note: moved it at top of emu.h, we need to define 'error' for priv.h 
   #define error(f,a...)	 	fprintf(stderr, f, ##a) */
 
 #define hard_error(f, a...)	fprintf(stderr, f, ##a)
 #define dbug_printf(f,a...)	ifprintf(2,f,##a)
+#define flush_log()		{ if (dbg_fd) log_printf(-1, "\n"); }
 
 #ifndef NO_DEBUGPRINT_AT_ALL
 
-#define k_printf(f,a...) 	(d.keyb?ifprintf(d.keyb,f,##a):0)
+#define k_printf(f,a...) 	ifprintf(d.keyb,f,##a)
 #define h_printf(f,a...) 	ifprintf(d.hardware,f,##a)
 #define v_printf(f,a...) 	ifprintf(d.video,f,##a)
 #define X_printf(f,a...)        ifprintf(d.X,f,##a)
@@ -224,6 +204,7 @@ void p_dos_str(char *,...) FORMAT(printf, 1, 2);
 #define p_printf(f,a...) 	ifprintf(d.printer,f,##a)
 #define d_printf(f,a...) 	ifprintf(d.disk,f,##a)
 #define i_printf(f,a...) 	ifprintf(d.io,f,##a)
+#define T_printf(f,a...) 	ifprintf(d.io_trace,f,##a)
 #define R_printf(f,a...) 	ifprintf(d.read,f,##a)
 #define W_printf(f,a...) 	ifprintf(d.write,f,##a)
 #define C_printf(f,a...)        ifprintf(d.cdrom,f,##a)
@@ -240,6 +221,7 @@ void p_dos_str(char *,...) FORMAT(printf, 1, 2);
 #define r_printf(f,a...)        ifprintf(d.request,f,##a)
 #define warn(f,a...)     	ifprintf(d.warning,f,##a)
 #define S_printf(f,a...)     	ifprintf(d.sound,f,##a)
+#define ds_printf(f,a...)     	ifprintf(d.dos,f,##a)
 
 #else
 
@@ -267,6 +249,7 @@ void p_dos_str(char *,...) FORMAT(printf, 1, 2);
 #define r_printf(f,a...)
 #define warn(f,a...)
 #define S_printf(f,a...)
+#define ds_printf(f,a...)
 
 #endif
 
@@ -319,6 +302,8 @@ EXTERN struct ioctlq curi INIT({0, 0, 0, 0});
 	  errno, strerror(errno), __FILE__, __LINE__, #sc); \
   s_tmp; })
 
+#define SILENT_DOS_SYSCALL(sc) sc
+
 #if 0
 #define RPT_SYSCALL(sc) ({ int s_tmp, s_err; \
    do { \
@@ -440,6 +425,7 @@ EXTERN struct ioctlq curi INIT({0, 0, 0, 0});
        unsigned int hogthreshold;
 
        int mem_size, xms_size, ems_size, dpmi, max_umb;
+       int secure;
        unsigned int ems_frame;
        char must_spare_hardware_ram;
        char hardware_pages[ ((HARDWARE_RAM_STOP-HARDWARE_RAM_START) >> 12)+1 ];
@@ -455,6 +441,7 @@ EXTERN struct ioctlq curi INIT({0, 0, 0, 0});
        unsigned char *num_table;
        unsigned short detach;
        unsigned char *debugout;
+       unsigned char *pre_stroke;  /* pointer to keyboard pre strokes */
 
        /* Lock File business */
        char *tty_lockdir;	/* The Lock directory  */
@@ -500,6 +487,9 @@ config_t;
 #define KEYB_PO               20
 #define KEYB_IT               21
 #define KEYB_SW		      22
+#define KEYB_HU		      23
+#define KEYB_HU_CWI	      24
+#define KEYB_HU_LATIN2	      25
 /*
  * Right now, dosemu only supports two serial ports.
  */
@@ -608,7 +598,7 @@ EXTERN void sigio(int, int, struct sigcontext *);
 					sigaddset(&sa.sa_mask, SIGNALS_THAT_QUEUE); \
 					sigaction(sig, &sa, NULL);
 
-#define SETSIG(sig, fun)	sa.sa_handler = (SignalHandler)fun; \
+#define SETSIG(sig, fun)	sa.sa_handler = (__sighandler_t)fun; \
 					sa.sa_flags = SA_RESTART; \
 					sigemptyset(&sa.sa_mask); \
 					sigaddset(&sa.sa_mask, SIG_TIME); \
@@ -622,6 +612,9 @@ EXTERN void sigio(int, int, struct sigcontext *);
 			sigaction(sig, &sa, NULL);
 #endif
 #ifdef __linux__
+#ifdef __GLIBC__
+#define SignalHandler __sighandler_t
+#endif
 #define NEWSETQSIG(sig, fun)	sa.sa_handler = (__sighandler_t)fun; \
 			/* Point to the top of the stack, minus 4 \
 			   just in case, and make it aligned  */ \
@@ -654,10 +647,6 @@ EXTERN inline void SIGNAL_save( void (*signal_call)(void) );
 EXTERN inline void handle_signals(void);
 
 
-#ifdef REQUIRES_VM86PLUS
-  #include "emusys.h"
-#endif
-
 /* 
  * DANG_BEGIN_REMARK
  * DOSEMU keeps system wide configuration status in a structure
@@ -673,7 +662,7 @@ EXTERN struct config_info config;
  * exit on the next return from vm86 mode.
  * DANG_END_REMARK
  */
-EXTERN int fatalerr;
+EXTERN int fatalerr INIT(0);
 
 #ifdef __NetBSD__
 #define iopl(value)			{ EXTERN int errno; if (i386_iopl(value) == -1) { g_printf("iopl failed: %s\n", strerror(errno)); leavedos(4);}}
